from pycryptopp.cipher import aes, xsalsa20

from common import insecurerandstr, rep_bench

UNITS_PER_SECOND = 10**9

class BenchCrypt(object):
    def __init__(self, klass, keysize):
        self.klass = klass
        self.keysize = keysize

    def __repr__(self):
        return "<%s-%d>" % (self.klass.__name__, self.keysize*8)

    def crypt_init(self, N):
        self.msg = insecurerandstr(N)
        self.key = insecurerandstr(self.keysize)

    def crypt(self, N):
        cryptor = self.klass(self.key)
        cryptor.process(self.msg)
        
def bench_ciphers(MAXTIME):
    for (klass, keysize) in [
        (aes.AES, 16),
        (aes.AES, 32),
        (xsalsa20.XSalsa20, 32),
        ]:
        ob = BenchCrypt(klass, keysize)
        print ob
        for (legend, size) in [
            ("large (%d B)",  10**7),
            ]:
            print legend % size
            rep_bench(ob.crypt, size, UNITS_PER_SECOND=UNITS_PER_SECOND, MAXTIME=MAXTIME, MAXREPS=100, initfunc=ob.crypt_init)
            print

    print "nanoseconds per byte crypted"
    print

def bench(MAXTIME=10.0):
    bench_ciphers(MAXTIME)

if __name__ == '__main__':
    bench()



import bench_sigs, bench_ciphers, bench_hashes

def bench(MAXTIME=10.0):
    bench_sigs.bench(MAXTIME)
    bench_ciphers.bench(MAXTIME)
    bench_hashes.bench(MAXTIME)

if __name__ == '__main__':
    bench()



msg = "crypto libraries should come with benchmarks"

try:
    import pyutil.benchutil
    rep_bench = pyutil.benchutil.rep_bench
except (ImportError, AttributeError):
    import platform, time
    if 'windows' in platform.system().lower():
        clock = time.clock
    else:
        clock = time.time

    def this_rep_bench(func, N, UNITS_PER_SECOND, MAXTIME, MAXREPS, initfunc=None):
        tt = time.time

        if initfunc is not None:
            initfunc(N)

        meanc = 0
        MAXREPS = 100

        timeout = tt() + MAXTIME

        for i in range(MAXREPS):
            startc = clock()

            func(N)

            stopc = clock()

            deltac = stopc - startc
            if deltac <= 0:
                print "clock jump backward or wrapped -- ignoring this sample. startc: %s, stopc: %s, deltac: %s" % (startc, stopc, deltac,)
            else:
                meanc += deltac

            if time.time() >= timeout:
                break

        num = i+1
        meanc *= UNITS_PER_SECOND
        meanc /= num
        meanc /= N

        res = {
            'meanc': meanc,
            'num': num
            }
        print "mean: %(meanc)#8.03e (of %(num)6d)" % res
    rep_bench = this_rep_bench

import random as insecurerandom
def insecurerandstr(n):
    return ''.join(map(chr, map(insecurerandom.randrange, [0]*n, [256]*n)))

def calib_clock():
    interval = 1.0
    
    import time
    tc = time.clock
    tt = time.time

    def measure_sleep(x, clock):
        st = clock()
        time.sleep(x)
        
    



from keys import (BadSignatureError, SigningKey, VerifyingKey, __doc__)

(BadSignatureError, SigningKey, VerifyingKey, __doc__) # hush pyflakes

from _version import get_versions
__version__ = get_versions()['version']
del get_versions



from pycryptopp.hash import sha256

from common import insecurerandstr, rep_bench

UNITS_PER_SECOND = 10**9

class SHA256(object):
    def proc_init(self, N):
        self.msg = insecurerandstr(N)

    def proc(self, N):
        h = sha256.SHA256()
        h.update(self.msg)
        h.digest()

def generate_hash_benchers():
    try:
        import hashlib
    except ImportError:
        return [SHA256]
    else:
        class hashlibSHA256(object):
            def proc_init(self, N):
                self.msg = insecurerandstr(N)

            def proc(self, N):
                h = hashlib.sha256()
                h.update(self.msg)
                h.digest()
                
        return [SHA256, hashlibSHA256]
    
def bench_hashes(MAXTIME):
    for klass in generate_hash_benchers():
        print klass
        ob = klass()
        for (legend, size) in [
            ("large (%d B)",  10**7),
            ]:
            print legend % size
            rep_bench(ob.proc, size, UNITS_PER_SECOND=UNITS_PER_SECOND, MAXTIME=MAXTIME, MAXREPS=100, initfunc=ob.proc_init)
            print

    print "nanoseconds per byte hashed"
    print


def bench(MAXTIME=10.0):
    bench_hashes(MAXTIME)

if __name__ == '__main__':
    bench()



from pycryptopp.publickey import ecdsa, ed25519, rsa

from common import insecurerandstr, rep_bench

msg = 'crypto libraries should come with benchmarks'

class ECDSA256(object):
    def __init__(self):
        self.seed = insecurerandstr(32)
        self.signer = None

    def gen(self, N):
        for i in xrange(N):
             ecdsa.SigningKey(self.seed)
        
    def sign_init(self, N):
        self.signer = ecdsa.SigningKey(self.seed)
        
    def sign(self, N):
        signer = self.signer
        for i in xrange(N):
            signer.sign(msg)
        
    def ver_init(self, N):
        signer = ecdsa.SigningKey(self.seed)
        self.sig = signer.sign(msg)
        self.verifier = signer.get_verifying_key()
        
    def ver(self, N):
        sig = self.sig
        verifier = self.verifier
        for i in xrange(N):
            verifier.verify(sig, msg)
        
class Ed25519(object):
    def __init__(self):
        self.seed = insecurerandstr(32)
        self.signer = None

    def gen(self, N):
        for i in xrange(N):
             ed25519.SigningKey(self.seed)
        
    def sign_init(self, N):
        self.signer = ed25519.SigningKey(self.seed)
        
    def sign(self, N):
        signer = self.signer
        for i in xrange(N):
            signer.sign(msg)
        
    def ver_init(self, N):
        signer = ed25519.SigningKey(self.seed)
        self.sig = signer.sign(msg)
        self.verifier = ed25519.VerifyingKey(signer.get_verifying_key_bytes())
        
    def ver(self, N):
        sig = self.sig
        verifier = self.verifier
        for i in xrange(N):
            verifier.verify(sig, msg)
        
class RSA2048(object):
    SIZEINBITS=2048

    def __init__(self):
        self.signer = None

    def gen(self, N):
        for i in xrange(N):
             rsa.generate(sizeinbits=self.SIZEINBITS)
        
    def sign_init(self, N):
        self.signer = rsa.generate(sizeinbits=self.SIZEINBITS)
        
    def sign(self, N):
        signer = self.signer
        for i in xrange(N):
            signer.sign(msg)
        
    def ver_init(self, N):
        signer = rsa.generate(sizeinbits=self.SIZEINBITS)
        self.sig = signer.sign(msg)
        self.verifier = signer.get_verifying_key()
        
    def ver(self, N):
        sig = self.sig
        verifier = self.verifier
        for i in xrange(N):
            verifier.verify(msg, sig)
        
class RSA3248(object):
    SIZEINBITS=3248

    def __init__(self):
        self.signer = None

    def gen(self, N):
        for i in xrange(N):
             rsa.generate(sizeinbits=self.SIZEINBITS)
        
    def sign_init(self, N):
        self.signer = rsa.generate(sizeinbits=self.SIZEINBITS)
        
    def sign(self, N):
        signer = self.signer
        for i in xrange(N):
            signer.sign(msg)
        
    def ver_init(self, N):
        signer = rsa.generate(sizeinbits=self.SIZEINBITS)
        self.sig = signer.sign(msg)
        self.verifier = signer.get_verifying_key()
        
    def ver(self, N):
        sig = self.sig
        verifier = self.verifier
        for i in xrange(N):
            verifier.verify(msg, sig)
        
def bench_sigs(MAXTIME):
    for klass in [ECDSA256, Ed25519,]:
        print klass
        ob = klass()
        print "generate key"
        rep_bench(ob.gen, 1000, UNITS_PER_SECOND=1000, MAXTIME=MAXTIME, MAXREPS=100)
        print "sign"
        rep_bench(ob.sign, 1000, UNITS_PER_SECOND=1000, initfunc=ob.sign_init, MAXTIME=MAXTIME, MAXREPS=100)
        print "verify"
        rep_bench(ob.ver, 1000, UNITS_PER_SECOND=1000, initfunc=ob.ver_init, MAXTIME=MAXTIME, MAXREPS=100)
        print

    for klass in [RSA2048, RSA3248]:
        print klass
        ob = klass()
        print "generate key"
        rep_bench(ob.gen, 1, UNITS_PER_SECOND=1000, MAXTIME=MAXTIME, MAXREPS=100)
        print "sign"
        rep_bench(ob.sign, 1000, UNITS_PER_SECOND=1000, initfunc=ob.sign_init, MAXTIME=MAXTIME, MAXREPS=100)
        print "verify"
        rep_bench(ob.ver, 10000, UNITS_PER_SECOND=1000, initfunc=ob.ver_init, MAXTIME=MAXTIME, MAXREPS=100)
        print

    print "milliseconds per operation"
    print

def bench(MAXTIME=10.0):
    bench_sigs(MAXTIME)

if __name__ == '__main__':
    bench()
